<?php

/**
 * The file that defines all functions which are to be performed
 * at admin side.
 *
 * This includes adding DivideBuy settings tab in Woocommerce -> Settings -> DivideBuy.
 */

if ( ! defined( 'ABSPATH' ) ) { 
    exit; // Exit if file accessed directly
}

class WC_Dividebuy_Settings_Tab {
    
    /**
     * Initializing hooks for settings tab.
     */
    public static function init() {
        add_filter( 'woocommerce_settings_tabs_array', __CLASS__ . '::add_settings_tab', 50 );
        add_action( 'woocommerce_settings_tabs_dividebuy', __CLASS__ . '::dividebuy_settings_tab' );
        add_action( 'woocommerce_update_options_dividebuy', __CLASS__ . '::update_dividebuy_settings' );
    }    
    
    /**
     * Adding a new settings tab to the woocommerce settings tab array.
     */
    public static function add_settings_tab( $settings_tabs ) {
        $settings_tabs['dividebuy'] = __( 'DivideBuy', 'divide-buy' );
        return $settings_tabs;
    }
    
    /**
     * Using woocommerce admin fields API to output settings via woocommerce_admin_fields() function.
     */
    public static function dividebuy_settings_tab() {
        woocommerce_admin_fields( self::get_dividebuy_settings() );
    }
    
    /**
     * Using woocommerce options API to save settings via woocommerce_update_options() function.
     */
    public static function update_dividebuy_settings() {
        $retalier_image_wc = get_option('wc_settings_tab_retailer_image');
        $check_image_wc = get_option('wc_settings_tab_dividebuy_checkout_banner');
        $check_hover_image_wc = get_option('wc_settings_tab_dividebuy_checkout_banner_hover');
        $wc_settings_tab_dividebuy_logo = get_option('wc_settings_tab_dividebuy_logo');
        
        // if($retalier_image_wc != $_REQUEST['wc_settings_tab_retailer_image']){
        //     DivideBuy_Helper::dividebuy_get_image_id($retalier_image_wc); 
        // }
        // if($check_image_wc != $_REQUEST['wc_settings_tab_dividebuy_checkout_banner']){
        //     DivideBuy_Helper::dividebuy_get_image_id($check_image_wc); 
        // }
        // if($check_hover_image_wc != $_REQUEST['wc_settings_tab_dividebuy_checkout_banner_hover']){
        //     DivideBuy_Helper::dividebuy_get_image_id($check_hover_image_wc); 
        // }
        // if($wc_settings_tab_dividebuy_logo != $_REQUEST['wc_settings_tab_dividebuy_logo']){
        //     DivideBuy_Helper::dividebuy_get_image_id($wc_settings_tab_dividebuy_logo); 
        // }

        $get_environment = get_option('wc_settings_tab_dividebuy_environment');
        woocommerce_update_options( self::get_dividebuy_settings() );
        $new_enviroment =$_REQUEST['wc_settings_tab_dividebuy_environment'];
        if($get_environment == $new_enviroment)
        {
            $post_data_array = json_encode(array('storeToken' => get_option('wc_settings_tab_retailer_token_number'), 'storeAuthentication' => get_option('wc_settings_tab_retailer_authentication_number'), 'retailerStoreCode' => ''));    
        }
        else
        {
            $post_data_array = json_encode(array('storeToken' => get_option('wc_settings_tab_retailer_token_number'), 'storeAuthentication' => get_option('wc_settings_tab_retailer_authentication_number'), 'retailerStoreCode' => '','CallRetailer'=>1));                
        }
        
        $url = Divide_Buy_Configuration::get_api_url().'api/confirmretailer';
        $method = 'PUT';
        $retrieved_data = DivideBuy_Api::curl_call_api($url,$post_data_array,$method);        
        $retrieved_data = json_decode($retrieved_data);
        update_option('wc_settings_tab_retailer_id', $retrieved_data->retailerId);
    }
    /**
     * Getting all settings fields for woocommerce_admin_fields() function.
     */
    public static function get_dividebuy_settings() {
        $settings = array(
            // General settings tab
            'dividebuy_general_section_title' => array(
                'name'      => __( 'General Settings', 'divide-buy' ),
                'type'      => 'title',
                'desc'      => 'Manage all your general settings for DivideBuy.',
                'id'        => 'wc_settings_tab_dividebuy_general_section_title'
            ),
            'dividebuy_retailer_name' => array(
                'name'      => __( 'Store Name', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Enter name of retailer.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_retailer_name'
            ),
            'dividebuy_token_number' => array(
                'name'      => __( 'Token Number', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Token number of retailer.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_retailer_token_number'
            ),
            'dividebuy_authentication_number' => array(
                'name'      => __( 'Authentication', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Authentication number of retailer.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_retailer_authentication_number'
            ),
            'wc_settings_tab_allowed_ip' => array(
                'name'      => __( 'Allowed IPs', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Display dividedbuy option for mentioned ip(For multiple ip use , seprator).', 'divide-buy' ),
                'id'        => 'wc_settings_tab_allowed_ip'
            ),
            'dividebuy_environment' => array(
                'name'      => __( 'Environment', 'divide-buy' ),
                'type'      => 'select',
                'options'   => array('Staging', 'Production'),
                'class'     => 'wc-enhanced-select',
                'desc'      => __( 'Select DivideBuy environment.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_environment'
            ),
            'dividebuy_errorlog' => array(
                'name'      => __( 'Error log', 'divide-buy' ),
                'type'      => 'select',
                'options'   => array('No', 'Yes'),
                'class'     => 'wc-enhanced-select',
                'desc'      => __( 'Select DivideBuy error log status.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_errorlog'
            ),
            'dividebuy_checkout_url_prefix' => array(
                'name'      => __( 'URL Prefix', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'This will be set while redirecting to DivideBuy checkout page.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_checkout_url_prefix'
            ),
            'dividebuy_retailer_image' => array(
                'name'      => __( 'Retailer Image', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Enter URL of retailer image.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_retailer_image'
            ),
            'dividebuy_retailer_id' => array(
                'name'      => __( 'Retailer Id', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_retailer_id'
            ),
            'dividebuy_retailer_auto_shipping' => array(
                'name'      => __( 'Auto shipping Method', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_auto_shipping'
            ),
            'dividebuy_minimum_order_value' => array(
                'name'      => __( 'Minimum Order Value', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Minimum order value for DivideBuy payment.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_minimum_order_value'
            ),
        'dividebuy_checkout_postcode_list' => array(
                'name'      => __( 'List of undeliverable shipping postcode', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_checkout_postcode_list'
            ),
        'dividebuy_logo_url_retailer_api' => array(
                'name'      => __( 'Logo Url', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_logo_url_retailer_api'
            ),
            'dividebuy_tracking_carrier_list' => array(
                'name'      => __( 'Tracking Courier List', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_tracking_carrier_list'
            ),
            'dividebuy_checkout_tax_class' => array(
                'name'      => __( 'Tax Class', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_tax_class'
            ),
            'dividebuy_checkout_flag_to_delete_orders' => array(
                'name'      => __( 'Flag To Delete Orders', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_flag_to_delete_orders'
            ),
            'dividebuy_retailer_installments' => array(
                'name'      => __( 'Installment Details', 'divide-buy' ),
                'type'      => 'hidden',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_retailer_installments'
            ),
            'dividebuy_general_section_end' => array(
                'type'      => 'sectionend',
                'id'        => 'wc_settings_tab_dividebuy_general_section_end'
            ),
            
            // Product detail page settings tab
            'dividebuy_product_section_title' => array(
                'name'      => __( 'Product Page Settings', 'divide-buy' ),
                'type'      => 'title',
                'desc'      => 'Manage all your product details page settings for DivideBuy.',
                'id'        => 'wc_settings_tab_dividebuy_product_section_title'
            ),
            'dividebuy_enable_logo' => array(
                'name'      => __( 'Enable DivideBuy Banner', 'divide-buy' ),
                'type'      => 'checkbox',
                'desc'      => __( 'Check this if you want to show DivideBuy banner.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_enable_logo'
            ),
            'dividebuy_logo_url' => array(
                'name'      => __( 'Banner Image', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'desc'      => __( 'Please note that image will get displayed only if banner has been enabled.', 'divide-buy' ),
                'id'        => 'wc_settings_tab_dividebuy_logo'
            ),
            'dividebuy_logo_custom_css' => array(
                'name'      => __( 'Custom CSS', 'divide-buy' ),
                'type'      => 'textarea',
                'class'     => 'input-text wide-input',
                'id'        => 'wc_settings_tab_dividebuy_logo_custom_css'
            ),
            'dividebuy_product_section_end' => array(
                'type'      => 'sectionend',
                'id'        => 'wc_settings_tab_dividebuy_product_section_end'
            ),
            
            // Cart page settings tab
            'dividebuy_cart_section_title' => array(
                'name'      => __( 'Cart Page Settings', 'divide-buy' ),
                'type'      => 'title',
                'desc'      => 'Manage all your cart settings for DivideBuy.',
                'id'        => 'wc_settings_tab_dividebuy_cart_section_title'
            ),
            'dividebuy_cart_banner_position' => array(
                'name'      => __( 'Where you want to show?', 'divide-buy' ),
                'type'      => 'select',
                'options'   => array('After Proceed to Checkout Button', 'Before Proceed to Checkout Button'),
                'class'     => 'wc-enhanced-select',
                'desc'      => 'Set banner position for DivideBuy.',
                'id'        => 'wc_settings_tab_dividebuy_cart_banner_position'
            ),
            'dividebuy_checkout_banner_url' => array(
                'name'      => __( 'Button Image', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_checkout_banner'
            ),
            'dividebuy_checkout_banner_url_hover' => array(
                'name'      => __( 'Button Image (Hover)', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_checkout_banner_hover'
            ),
            'dividebuy_checkout_banner_custom_css' => array(
                'name'      => __( 'Custom CSS', 'divide-buy' ),
                'type'      => 'textarea',
                'class'     => 'input-text wide-input',
                'id'        => 'wc_settings_tab_dividebuy_banner_custom_css'
            ),
            'dividebuy_checkout_banner_prefix_label' => array(
                'name'      => __( 'Button Prefix Label', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_checkout_banner_prefix_label'
            ),
            'dividebuy_checkout_banner_prefix_label_css' => array(
                'name' => __('Button Prefix Label CSS', 'divide-buy'),
                'type' => 'text',
                'class' => 'input-text regular-input',
                'id' => 'wc_settings_tab_dividebuy_checkout_banner_prefix_label_css'
            ),
            'dividebuy_checkout_banner_suffix_label' => array(
                'name'      => __( 'Button Suffix Label', 'divide-buy' ),
                'type'      => 'text',
                'class'     => 'input-text regular-input',
                'id'        => 'wc_settings_tab_dividebuy_checkout_banner_sufffix_label'
            ),
            'dividebuy_checkout_banner_sufffix_label_css' => array(
                'name' => __('Button Suffix Label CSS', 'divide-buy'),
                'type' => 'text',
                'class' => 'input-text regular-input',
                'id' => 'wc_settings_tab_dividebuy_checkout_banner_sufffix_label_css'
            ),
            'dividebuy_enable_coupon_code' => array(
                'name'      => __( 'Enable checkout with coupon code', 'divide-buy' ),
                'type'      => 'checkbox',
                'desc'      => __( 'Check this if you want to allow user to checkout with DivideBuy using coupon code.', 'divide-buy' ),
                'id'        => 'dividebuy_enable_coupon_code'
            ),
            'dividebuy_cart_section_end' => array(
                'type'      => 'sectionend',
                'id'        => 'wc_settings_tab_dividebuy_cart_section_end'
            ),
        );
        return apply_filters( 'wc_settings_tab_dividebuy', $settings );
    }

}

WC_Dividebuy_Settings_Tab::init();
