<?php

/**
 * The file that defines all functions which are to be performed
 * at admin side.
 *
 * It includes all function related to order management.
 */

if ( ! defined( 'ABSPATH' ) ) { 
    exit; // Exit if file accessed directly
}

/**
 * Registering new order status for processing orders from DivideBuy side.
 * 
 * In case order has been put for hold due to guarantor then below status will be set to order.
 */

// Registering new order status
add_action( 'init', 'register_new_order_status_for_freeze_orders_dividebuy' );

function register_new_order_status_for_freeze_orders_dividebuy() {
    
    register_post_status( 'wc-freeze-dividebuy', array(
        'label'                     => _x( 'Freeze from DivideBuy', 'Order status', 'woocommerce' ),
        'public'                    => true,
        'exclude_from_search'       => true,
        'show_in_admin_all_list'    => false,
        'show_in_admin_status_list' => false,
        'label_count'               => _n_noop( 'Freeze from DivideBuy <span class="count">(%s)</span>', 'Freeze from DivideBuy<span class="count">(%s)</span>', 'woocommerce' )
    ) );
}

add_filter( 'wc_order_statuses', 'add_freeze_order_status_in_status_list' );

// Register status in wc_order_status.
function add_freeze_order_status_in_status_list( $order_statuses ) {
    global $post;
    $order_is_dividebuy = get_post_meta( $post->ID, 'order_is_dividebuy')[0];
    if( $order_is_dividebuy == "yes" ){
        unset( $order_statuses['wc-cancelled'] );
    }
    $order_statuses['wc-freeze-dividebuy'] = _x( 'Freeze from DivideBuy', 'Order status', 'woocommerce' );

    return $order_statuses;
}

add_action( 'add_meta_boxes', 'add_order_meta_boxes' );

// Adding order meta box for storing order tracking number.
function add_order_meta_boxes() {
    add_meta_box( 
        'woocommerce-track-order', 
        __( 'Order Tracking' ), 
        'track_order', 
        'shop_order', 
        'side', 
        'default' 
    );
}

// Displaying field for order tracking number.
function track_order($post) {
    global $wpdb;
    $dividebuy_tracking_table = $wpdb->prefix . "dividebuy_tracking";
    
    // Getting all tracking details of an order and displaying the same.
    $tracking_data = $wpdb->get_results( "SELECT * FROM $dividebuy_tracking_table WHERE order_id = '".$post->ID."' ORDER BY id DESC" );
    if(count($tracking_data) > 0){
        echo '<div class="tracking_details">';
        foreach ($tracking_data as $data){
            echo '<div class="tracking_content">';
            echo '<span>Tracking Carrier: '.$data->tracking_carrier.'</span><br/>';
            echo '<span>Tracking Number: '.$data->tracking_number.'</span><br/>';
            echo '</div>';
            echo '<p class="meta"><abbr class="exact-date">Added on '.$data->created_date.' at '.$data->created_time.' </abbr>';
            echo '<a href="#" rel="'.$data->id.'" class="delete_note" >Delete</a></p>';
        }
        echo '</div>';
    }
    
    $values = get_post_custom( $post->ID );
    $tracking_carrier = isset( $values['order_tracking_carrier'] ) ? esc_attr( $values['order_tracking_carrier'][0] ) : "";
    $tracking_number = isset( $values['order_tracking_number'] ) ? esc_attr( $values['order_tracking_number'][0] ) : "";
    $is_order_dividebuy = isset( $values['order_is_dividebuy'] ) ? esc_attr( $values['order_is_dividebuy'][0] ) : "";        
    wp_nonce_field( 'order_tracking_nonce', 'meta_box_nonce' );
    
    // Getting all available tracking carriers.
    $tracking_carrier_list = get_option('wc_settings_tab_dividebuy_tracking_carrier_list');
    $tracking_carrier_list = explode(',', $tracking_carrier_list);
    if(count($tracking_carrier_list) > 0){ ?>
        <datalist id="carriers">
            <?php
            foreach ($tracking_carrier_list as $list){
                echo '<option value="'.$list.'">';
            }
            ?>
        </datalist>
    <?php } ?>
    
    <!--Form for adding tracking information for an order.-->
    <form method="post">
        <label for="order_tracking_carrier"><?php esc_html_e( 'Tracking Carrier', 'woocommerce' )?></label>
        <input type="text" name="order_tracking_carrier" id="order_tracking_carrier" list="carriers" value="<?php echo $tracking_carrier;?>"/>

        <label for="order_tracking_number"><?php esc_html_e( 'Tracking Number', 'woocommerce' )?></label>
        <input type="text" name="order_tracking_number" id="order_tracking_number" value="<?php echo $tracking_number;?>"/>
        <input type="hidden" name="order_is_dividebuy" id="order_is_dividebuy" value="<?php echo $is_order_dividebuy;?>" />
        <button type="submit" name="add_new_tracking" class="add_tracking button" id="add_new_tracking"><?php _e( 'Add', 'woocommerce' ); ?></button>
    </form>
    
    <!--Adding script for deleting tracking information-->
    <script>
        jQuery("#add_new_tracking").click(function(){
            if(jQuery("#order_tracking_carrier").val() === ""){
                alert("Please enter tracking carrier");
                return false;
            }
            if(jQuery("#order_tracking_number").val() === ""){
                alert("Please enter tracking number");
                return false;
            }
        });
        jQuery(".delete_note").click(function(){
            var confirm_delete = confirm("Are you sure you wish to delete tracking information? This action cannot be undone.");
            if (confirm_delete === true) {
                jQuery(this).text("Deleting...");
                jQuery.ajax({
                    type: 'post',
                    url: '<?php echo plugin_dir_url( __FILE__ ) . 'order-tracking.php';?>',
                    data: {post_id: <?php echo $post->ID; ?>,delete_tracking_id: jQuery(this).attr('rel')},
                    success: function(response){
                        if(response === "success"){
                            alert("Tracking information has been updated.");
                            window.location.reload();
                        }else{
                            alert("There is some problem in updating information.");
                            window.location.reload();
                        }
                    },
                    error: function () {
                        alert("There is some error, Please try again later.");
                        window.location.reload();
                    }
                });
            } else {
                return false;
            }                    
        });
    </script>
    <?php
}

// Saving order meta box.
add_action( 'save_post', 'save_tracking_data' );
function save_tracking_data( $post_id ) {
    if( get_post_type() == "shop_order" ){
        global $wpdb;
        $dividebuy_tracking_table = $wpdb->prefix . "dividebuy_tracking";
        $delete_tracking = 0;
        $field_empty = 0;

        // Bail if we're doing an auto save
        if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) return;

        // if our current user can't edit this post, bail
        if( !current_user_can( 'edit_post' ) ) return;

        // Getting value of entered data.
        $order_tracking_carrier = $_POST['order_tracking_carrier'];
        $order_tracking_number = $_POST['order_tracking_number'];
        
        if((trim($order_tracking_carrier) == "") || (trim($order_tracking_number) == "")){
            $field_empty = 1;
        }

        $dividebuy_helper = new DivideBuy_Helper();
        $tracking_data = json_decode($dividebuy_helper->send_tracking_number_information($post_id, $order_tracking_number, $order_tracking_carrier, $delete_tracking));
        
        // Adding tracking information if there is no error.
        if($tracking_data->error == 0 && $field_empty == 0){
            $wpdb->insert( 
                $dividebuy_tracking_table, 
                array( 
                    'order_id'          => $post_id, 
                    'tracking_carrier'  => $order_tracking_carrier,
                    'tracking_number'   => $order_tracking_number,
                    'created_date'      => date("Y-m-d"),
                    'created_time'      => date("H:i:s")
                ), 
                array( 
                    '%d',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                )
            );
        }
    }
}
/*
* Don't allow admin to delete the dividebuy orders details only
*/
function hideMovetotrash()
{

    ?>
    <style type="text/css">
    #delete-action{display: none;}
    </style>
    <?php

}
/*
* Hide details trash on order details section
*/
function my_wp_trash_post () {
        if( !is_admin() )
            return;
    if ( isset($_GET['post']) ) {
        global $pagenow;
        if( in_array( $pagenow, array('post.php') ) ){
            $method  = get_post_meta( $_GET['post'] ,'_payment_method',true);
            if($method == 'dividebuy')
            {

                hideMovetotrash();

            }
        }
    }
}
add_action('init','my_wp_trash_post');
/*
* For Multiple bulkaction or single action of trash
*/
add_action( 'admin_action_trash', 'dividebuy_bulk_process_custom_status' ); // admin_action_{action name}

function dividebuy_bulk_process_custom_status() {
    
    //checking it is admin or not 
    if( !is_admin() )
        return;
    // if an array with order IDs is not presented, exit the function
    if( !isset( $_REQUEST['post'] )) return;
    $i = 0;
    foreach( $_REQUEST['post'] as $order_id ) {
        $method  = get_post_meta( $order_id ,'_payment_method',true);
        
        if($method == 'dividebuy')
        {
            $i++;
        }else
        {
            wp_trash_post( $order_id );
        }
    }
    if($i > 0)
    {
        set_transient( 'fx-admin-notice-panel', true, 5 );
        add_filter( 'map_meta_cap','mappingfunction', 10, 4 );

    }else
    {
        return;
    }
    global $pagenow;
    
    wp_redirect( admin_url( $pagenow.'?post_type=shop_order' ) );
    exit;
}
/*
*Remove Trash option
*/
add_filter( 'post_row_actions', 'remove_row_actions', 10, 2 );
function remove_row_actions( $actions, $post )
{
  global $current_screen;
    unset( $actions['trash'] );
    return $actions;
}

/*
* Stopping admin to update the status
*/
function mappingfunction ( $caps, $cap, $user_id, $args )
{
    $caps[] = 'do_not_allow';       
    return $caps;    
    
}

/*
 * Admin Notices
 */
add_action('admin_notices', 'dividebuy_custom_order_status_notices');
 
function dividebuy_custom_order_status_notices() {
    
    global $pagenow, $typenow;
    if( get_transient( 'fx-admin-notice-panel' )){
        echo "<div class=\"updated\"><p>Some orders are not changed as they are dividebuy orders.</p></div>";
    }
    delete_transient( 'fx-admin-notice-panel' );
}