<?php

/**
 * The file that includes functions for DivideBuy checkout modal on cart page.
 * 
 * @since    1.0
 * @package  Divide_Buy
 * @author   Cygnet Infotech Pvt. Ltd.
 */

class DivideBuy_Modal{
		
	/** 
         * Hook WordPress
	 * @return void
	 */
	public function __construct(){
            add_filter('query_vars', array($this, 'add_query_vars'), 0);
            add_action('parse_request', array($this, 'sniff_requests'), 0);
            add_action('init', array($this, 'add_endpoint'), 0);
	}	
	
	/** 
         * Add public query vars
	 * @param array $vars List of current public query vars
	 * @return array $vars 
	 */
	public function add_query_vars($vars){
            $vars[] = '__dividebuy';
            $vars[] = 'modal';
            $vars[] = 'user_login_action';
            $vars[] = 'guest';
            $vars[] = 'checkout';
            $vars[] = 'shipping_information';
            $vars[] = 'shipping_methods';
            $vars[] = 'place_order';
            $vars[] = 'update_checkout_cart';
            $vars[] = 'get_nondividebuy_count';
            return $vars;
	}
	
	/** 
         *  Add API Endpoint
	 *  @return void
	 */
	public function add_endpoint(){
            add_rewrite_rule('^dividebuy/modal/user_login_action?','index.php?__dividebuy&modal&user_login_action','top');
            add_rewrite_rule('^dividebuy/modal/guest?','index.php?__dividebuy&modal&guest','top');
            add_rewrite_rule('^dividebuy/modal/checkout?','index.php?__dividebuy&modal&checkout','top');
            add_rewrite_rule('^dividebuy/modal/shipping_information?','index.php?__dividebuy&modal&shipping_information','top');
            add_rewrite_rule('^dividebuy/modal/shipping_methods?','index.php?__dividebuy&modal&shipping_methods','top');
            add_rewrite_rule('^dividebuy/modal/place_order?','index.php?__dividebuy&modal&place_order','top');
            add_rewrite_rule('^dividebuy/modal/update_checkout_cart?','index.php?__dividebuy&modal&update_checkout_cart','top');
            add_rewrite_rule('^dividebuy/modal/get_nondividebuy_count?','index.php?__dividebuy&modal&get_nondividebuy_count','top');
            flush_rewrite_rules();
	}
        
	/**	
         * Sniff Requests
	 * This is where we hijack all requests
	 */
	public function sniff_requests(){
            global $wp;
            
            // User login modal
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['user_login_action'])){                
                $this->user_login_action();
            }
            
            // Continue as guest
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['guest'])){                
                $this->guest_login();
            }
            
            // Process to checkout modal
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['checkout'])){                
                $this->process_to_checkout();
            }
            
            // Shipping information modal
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['shipping_information'])){                
                $this->get_shpipping_information();
                exit;
            }
            
            // Shipping method listing
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['shipping_methods'])){                
                $this->get_shpipping_methods();
                exit;
            }
            
            // Placing new order when user click on checkout.
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['place_order'])){       
                $this->place_new_order();
                exit;
            }
            
            // Get non DivideBuy products count.
            if(isset($wp->query_vars['get_nondividebuy_count']) && isset($wp->query_vars['get_nondividebuy_count'])){       
                $non_dividebuy_products_count = DivideBuy_Helper::get_nondividebuy_products_count();
                echo $non_dividebuy_products_count;
                exit;
            }
            
            // if there are mixed products while checkout, remove non DivideBuy products and update cart.
            if(isset($wp->query_vars['modal']) && isset($wp->query_vars['update_checkout_cart'])){
                DivideBuy_Helper::remove_nondividebuy_products_from_cart();
                echo "true";
                exit;
            }
	}
        
        /**
         * Function for guest login.
         */
        public function guest_login(){
            
            WC()->session->set( 'dividebuy_guest_session', 1 );
            $this->process_to_checkout();
        }
        
        /**
         * 
         * Function for user login which generates user session.
         */
        public function user_login_action(){
            $email = filter_input(INPUT_POST, 'user_email');
            $password = filter_input(INPUT_POST, 'user_password');
            
            try{
                $user = get_user_by('email', $email);
                $user_valid = wp_check_password($password, $user->user_pass, $user->ID);
                if($user_valid){
                    wp_set_current_user( $user->ID, $user->user_login );
                    wp_set_auth_cookie( $user->ID );
                    do_action( 'wp_login', $user->user_login );
                    $this->process_to_checkout();
                }
            } catch (Exception $ex) {
                return 'failed';
            }          
            
            exit();
        }
        
        /**
         * Function for checkout process of DivideBuy modal.
         */
        public function process_to_checkout(){
            
            $nondividebuy_products_count = DivideBuy_Helper::get_nondividebuy_products_count();
            $user_logged_in = "false";            
            $guest_session = WC()->session->get('dividebuy_guest_session');
            
            // Storing shipping data in session.
            WC()->session->__unset('dividebuy_shipping');
            $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
            $chosen_shipping = explode(":", $chosen_methods[0]);
            $chosen_shipping_price = WC()->session->get( 'shipping_total' );
            $shipping = array($chosen_shipping[0], $chosen_shipping_price);
            WC()->session->set( 'dividebuy_shipping', $shipping );
            
            if(is_user_logged_in()){
                WC()->session->__unset('dividebuy_guest_session');
            }
            
            if( $guest_session || is_user_logged_in() ){
                if( $nondividebuy_products_count == 0 ){
                    $user_logged_in = "true_no_mixed_products";
                }else{
                    $user_logged_in = "true_with_mixed_products";
                }
            }
            
            switch ($user_logged_in){
                
                // If there are only DivideBuy products in cart, display shipping information.
                case "true_no_mixed_products":
                    $this->get_shpipping_information();
                    break;

                // If there are mixed products in cart, display seperated products in modal with grand total.
                case "true_with_mixed_products":
                    $this->get_all_cart_items();
                    break;

                // If user is neither logged in nor guest session, display login form.
                case "false":
                    $this->user_login_html();
                    break;
            }            
            exit;
        }
        
        /**
         * Function for getting list of all available shipping methods.
         */
        public function get_shpipping_methods(){
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/templates/shipping_methods.php';
        }
        
        /**
         * Function for user login.
         */
        public function user_login_html(){
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/templates/user_login.php';
        }
        
        /**
         * If there are mixed products in cart then displaying them in DivideBuy modal.
         */
        public function get_all_cart_items(){
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/templates/cart_items.php';
        }
        
        /**
         * Function for displaying postcode field.
         */
        public function get_shpipping_information(){
            DivideBuy_Helper::remove_nondividebuy_products_from_cart();
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/templates/shipping_information.php';
        }
        
        /**
         * Creates new custom order when user will click on checkout button.
         */
        public function place_new_order(){
            DivideBuy_Helper::create_new_order();
        }
}
new DivideBuy_Modal();